@@ -23,7 +23,7 @@ MAIN:
 
     # Get random numbers and put them in bins
     print("Generating $count Poisson random numbers.  Please wait...");
-    for (my $ii=0; $ii < $count; $ii++) {
+    for (1 .. $count) {
         my $x = poisson($mean);
 
         # Make sure the tail doesn't overflow
@@ -44,7 +44,7 @@ MAIN:
 
     # Output the graph
     print("\n");
-    foreach my $key (sort { $a <=> $b } (keys(%poisson))) {
+    for my $key (sort { $a <=> $b } (keys(%poisson))) {
         my $len = int(79.0 * $poisson{$key}/$max);
         print(':', '*' x $len, "\n");
     }
